#ifndef __DEVP2PLIB_P2P_H
#define __DEVP2PLIB_P2P_H

#ifdef _WIN32
#define int64 __int64
#else
#define int64 int64_t
#include <arpa/inet.h>
#endif

#include <vector>

namespace devP2Plib
{
	class CVPN;
	class CP2P;
	class CMediator;
	class CRelay;
	class CP2PForwardUser;
	class CVPNInterface;

	bool libInit(void);
	void upnpInit(void);
	void libFree(void);

	int VPNAdapterCount(void);
	CVPNInterface *VPNAdapter(int index);

	void TextToMac(char *src,unsigned char *dst);
	void MacToText(unsigned char *src, char dst[18]);


	enum Errors // public
	{
		ErrorNone = 0,
		ErrorStopped = -1,
		ErrorDisconnected = -2,
		ErrorTimeout = -3,
		ErrorNameNotSet = -4,
		ErrorCannotBind = -5,
		ErrorCancelled = -6,
		ErrorAborted = -7,
		ErrorDisconnectedByRemote = -8,
		ErrorConnectionRefused = -9,
		ErrorConnectionBroken = -10,
		ErrorChannelInvalid = -11,
		ErrorChannelFull = -12,
		ErrorChannelBusy = -13,
		ErrorCannotOpenFile = -14,
		ErrorCannotReadFile = -15,
		ErrorCannotWritefile = -16,
		ErrorForwardStarted = -17,
		ErrorForwardInvalid = -18,
		ErrorCannotOpenAdapter = -19,
		ErrorLicenseInvalid = -20,
		ErrorNameInvalid = -21,
		ErrorAdapterInvalid = -22,
		ErrorAdapterInvalidIP = -23,
		ErrorAdapterExistingIP = -24,
		ErrorLast = -99
	};

	enum Protocols { ProtoTCP = 1, ProtoUDP = 2, ProtoBoth = 3}; 
	enum Encryptions { EncNone = 0, EncAES128 = 1, EncAES192 = 2, EncAES256 = 3}; // public
	enum States { StateStopped = 0, StateStarted = 1, StateSearching = 2, StateSearchDone = 3, StateLinking = 4, StateConnected = 5, StateDisconnecting = 6, StateDisconnected = 7}; 
	enum ForwardTypes { TCPLocalListen = 0, TCPRemoteListen = 1, UDPLocalListen = 2, UDPRemoteListen = 3, TCPSocksProxy = 4, TCPHttpProxy = 5}; // public
	enum ConnectionTypes { ConnectionUnspecified = 0, ConnectionTCPDirect = 1, ConnectionUDPDirect = 2, ConnectionTCPRelayed = 3};

	typedef struct P2PEventsStruct
	{
		void (*StateChange)(CP2P *p2p, States state);
		void (*NewUPNPMapping)(CP2P *p2p, char *ExtAddress, char *IntAddress, int TCPPort, int UDPPort);
		void (*SearchStart)(CP2P *p2p, char *binds);
		void (*SearchDone)(CP2P *p2p, char *peerName, char *peerAddrs, char *customData, Errors error);
		void (*LinkDone)(CP2P *p2p, char *address, int port, Errors error);
		void (*Stopped)(CP2P *p2p, Errors error);
		void (*TextReceived)(CP2P *p2p, int chanid, char *text);
		void (*DataReceived)(CP2P *p2p, int chanid, char *data, int len);
		bool (*FileSend)(CP2P *p2p, int chanid, char *filename, int64 size);
		bool (*FileReceive)(CP2P *p2p, int chanid, char *filename, int64 size);
		void (*FileProgress)(CP2P *p2p, int chanid, int64 position, int64 size);
		void (*FileDone)(CP2P *p2p, int chanid, Errors error);
		bool (*ForwardOpen)(CP2P *p2p, int forwid, ForwardTypes forwardtype, char *localaddress, int localport, char *remoteaddress, int remoteport);
		void (*ForwardClose)(CP2P *p2p, int forwid, Errors error);
		bool (*UserConnected)(CP2P *p2p, int forwid, int chanid, CP2PForwardUser *user);
		void (*UserDisconnected)(CP2P *p2p, int forwid, int chanid, CP2PForwardUser *user, Errors error);
		void (*Ping)(CP2P *p2p);
	} P2PEventsStruct;

	typedef struct VPNEventsStruct
	{
		void (*StateChange)(CVPN *vpn, States state);
		bool (*PeerConnecting)(CVPN *vpn, CP2P *p2p, char *peerName, char *peerAddrs);
		void (*PeerConnected)(CVPN *vpn, CP2P *p2p, char *address, int port);
		void (*PeerDisconnected)(CVPN *vpn, CP2P *p2p, Errors error);
	} VPNEventsStruct;

	typedef struct MediatorEventsStruct
	{
		void (*StateChange)(CMediator *med, States state);
		bool (*PeerConnected)(CMediator *med, char *ip, int port, char *fromID, char *toID, char *Data);
		void (*PeerDisconnected)(CMediator *med, char *fromID, char *toID);
		bool (*PeerMatch)(CMediator *med, char *fromID, char *toID);
	} MediatorEventsStruct;

	typedef struct RelayEventsStruct
	{
		//		void (*StateChange)(CVPN *vpn, States state);
		//		bool (*PeerConnecting)(CVPN *vpn, CP2P *p2p, char *peerName, char *peerAddrs);
		//		void (*PeerConnected)(CVPN *vpn, CP2P *p2p, char *address, int port);
		//		void (*PeerDisconnected)(CVPN *vpn, CP2P *p2p, Errors error);
	} RelayEventsStruct;

	class CP2PForwardUser
	{
	public:
		CP2PForwardUser();
		virtual ~CP2PForwardUser();
		virtual void Disconnect(void) = 0;

		void *Tag;
#ifdef _WIN32
		LPDISPATCH m_pDisp;
#endif
	};


	class CP2PForward
	{
		friend class _CP2P;
	public:
		CP2PForward();
		virtual ~CP2PForward();
		virtual void DisconnectUser(CP2PForwardUser *user) = 0;
		virtual bool IsOpen(void) = 0;

		ForwardTypes ForwardType;
		char LocalAddress[1024];
		int LocalPort;
		char RemoteAddress[1024];
		int RemotePort;
		void *Tag;
#ifdef _WIN32
		LPDISPATCH m_pDisp;
#endif

	};

	class CVPN
	{
	protected:
		CVPN();
		virtual ~CVPN();

	public:
		// constructor
		static CVPN *Create(void);
		virtual void Destroy(void) = 0;

		char *MyName;
		char *RelayLicense;
		char *MediatorAddress;
		int MediatorPort;
		States State;
		void *Tag;
#ifdef _WIN32
		LPDISPATCH m_pDisp;
#endif
		VPNEventsStruct Events;
		std::vector<CP2P *> Peers;


		virtual int Start(Protocols prot = ProtoBoth) = 0;
		virtual void Stop(void) = 0;
		virtual int Search(char *peerNames, char *password, bool forever) = 0;
		virtual int SetAdapter(CVPNInterface *adapter) = 0;

		static const char *StateText(States state);
		static const char *ErrorText(Errors error);
		static const char *Version(void);

		virtual void SetLicenseKey(const char *license_key) = 0;
	};

	class CMediator
	{
	protected:
		CMediator();
		virtual ~CMediator();

	public:
		// constructor
		static CMediator *Create(void);
		virtual void Destroy(void) = 0;

		void *Tag;
#ifdef _WIN32
		LPDISPATCH m_pDisp;
#endif
		MediatorEventsStruct Events;
		States State;


		virtual int Start(int port) = 0;
		virtual void Stop(void) = 0;

		static const char *Version(void);

		virtual void SetLicenseKey(const char *license_key) = 0;
	};

	class CP2P
	{
	protected:
		CP2P();
		virtual ~CP2P();

	public:
		// constructor
		static CP2P *Create(void);
		virtual void Destroy(void) = 0;

		// properties
		int UDPPort, TCPPort;
		char *MyName, *PeerName;
		char *MediatorAddress;
		int MediatorPort;
		States State;
		unsigned char PeerAdapterMAC[6];
		unsigned int PeerAdapterIP;
		ConnectionTypes ConnectionType;
		void *Tag, *Tag1, *Tag2;
		unsigned int BufferSize;
#ifdef _WIN32
		LPDISPATCH m_pDisp;
#endif

		char *RelayLicense;

		P2PEventsStruct Events;

		// timeouts
		int MediatorRetryDelay, MediatorRetryCount;
		int LinkingRetryDelay, LinkingRetryCount, LinkingRelayDelay;
		int ConnectionTimeout;



		// methods
		virtual int Start(Protocols prot = ProtoBoth) = 0;
		virtual void Stop(void) = 0;
		virtual int Search(char *customdata) = 0;
		virtual int Link(char *peerAddrs) = 0;
		virtual void Disconnect(void) = 0;
		virtual void SetPassword(Encryptions encryption, char *password) = 0;
		virtual int SendText(int chanid, char *text, bool reliable = true) = 0;
		virtual int SendData(int chanid, char *data, int len, bool reliable = true) = 0;
		virtual int SendFile(int chanid, char *local_filename, char *remote_filename = 0, bool resume = false) = 0;
		virtual int Ping(void) = 0;
		virtual int TestBandwidth(int chanid, unsigned long size) = 0;
		virtual void PrintStats(void) = 0;
		virtual CP2PForward *GetForward(int index) = 0;
		virtual int StartForward(int index) = 0;
		virtual int StopForward(int index) = 0;
		virtual int SetAdapter(CVPNInterface *adapter) = 0;

		virtual void BandwidthReset(int chanid) = 0;
		virtual float Bandwidth(int chanid) = 0;

		static const char *StateText(States state);
		static const char *ErrorText(Errors error);
		static const char *Version(void);

		virtual void SetLicenseKey(const char *license_key) = 0;

	private:

	}; // end of class

	class CVPNInterface
	{
		public:
			CVPNInterface();
			virtual ~CVPNInterface();

			virtual void SetIP(char *ip, char *netmask) = 0;

			char Guid[1024], Name[1024];
			unsigned int LocalIP, LocalNetmask;
			unsigned char LocalMAC[6];

			void *Tag;
#ifdef _WIN32
			LPDISPATCH m_pDisp;
#endif

	};
}; // end of namespace

#endif // __DEVP2PLIB_P2P_H
